// ==UserScript==
// @name        HNhance
// @version     1.3
// @description Adds extra functions to Hacker News
// @match       http://news.ycombinator.com/*
// @match       https://news.ycombinator.com/*
// ==/UserScript==

/*

ABOUT

HNHance is a collection of assorted scripts to enhance Hacker News.

Feel free to modify and add scripts to it. I would like to keep everything done
 in JS where possible instead of relying on extra css, images, et al so that
 the script can be used either as a Chrome extension, or a Userscript in Firefox
 and Opera.

LICENSE

It's a userscript, not like you can make a saleable product out of this. Please
 attribute all modifications to the best of your ability, but most of all, create
 great new features!

ATTRIBUTIONS

- HNHance Cog icon from Glyphicons, via the Bootstrap project
- Modern Theme based on https://gist.github.com/kballenegger/5022493 - used
   without permission or notification seeing as nobody seems to know the original
   author
- Modern Theme background from Subtle Patterns
- Bundled "Georgify for HN" by @tuhin, from http://userstyles.org/styles/46180
- Bundled 4chan commentary script from an anonymous /g/ user, freely given with
   no specified license or contact information
- Bundled collapseComments script by @drostie <http://drostie.org/>, bundled
   under the terms of the Creative Commons Zero waiver/license (see below for
   details)

*/

(function() {
	"use strict";
	
	// Helpers
	// ```````
	
	var injectStyles = function(s) {
		var style = document.createElement("style");
		style.appendChild(document.createTextNode(s));
		document.documentElement.insertBefore(style);
	};
	
	var settings_get = function() {
		try {
			return JSON.parse(localStorage["hnhance"]);
		} catch(em_all) {
			return {
				'features': {
					'styleCommentLink': false,
					'hoverParents': true,
					'show4chan': true,
					'customStyle': true,
					'showSubmissionsLink': true,
					'confirmDownvotes': false,
					'collapseComments': false,
					'hoverProfiles': true
				},
				'theme': 'modern'
			};
		}
	};
	
	var settings_save = function(o) {
		localStorage["hnhance"] = JSON.stringify(o);
	};
	
	var settings_enum = {
		'styleCommentLink': 'Dim visited comment links',
		'hoverParents': 'Show parent hover links',
		'show4chan': 'Show related 4chan /g/ links',
		'showSubmissionsLink': 'Show submissions link',
		'confirmDownvotes': 'Confirm downvotes',
		'collapseComments': 'Allow collapsing threads',
		'hoverProfiles': 'Display user info on hover'
	};
	
	var settings_themes_enum = {
		'default': 'No custom theme',
		'modern': 'Modern',
		'georgify': 'GeorgifyMod'
	};
	
	/*!
	 * Feature: settingsDialog
	 * Always enabled.
	 */
	
	window.addEventListener("load", function() {
		
		injectStyles([
			".hnh-settings {",
				"display:inline-block;",
				"background:url('data:image/png;base64,",
					"iVBORw0KGgoAAAANSUhEUgAAAAwAAAAMCAYAAABWdVznAAAABHNCSVQICAgIf",
					"AhkiAAAAAlwSFlzAAALEgAACxIB0t1+/AAAABh0RVh0U29mdHdhcmUAQWRvYm",
					"UgRmlyZXdvcmtzT7MfTgAAAX5JREFUKJFtkTGLGlEUhc8d5LEyEZ+OxbswgrC","FWO3AINNIVphyktL/YpG/4D/wH1imSmcgxTKNU4QJgUDKh/A0syRCGI03Tdxs",
					"gre893znXDjAlWHmO2a+v3ajK+J2s9l863keHQ6HV9bax6sAM38DUHieN+12u",
					"wCA/X6P8/m8BhBZazsA0LgAw+Hw6263myqlntyMMajrehoEwcZa+zeBmdu9Xq",
					"9SSsH3/S/z+fwNACwWi3lVVVFd13DOaWvtI5j5PgzDdRzHEsexrFar1yJyIyI",
					"3eZ5PLvswDNfMfNdIkuTXdrv167oGACyXy8+z2ewnAGRZ9unyXr/f94mIISIv",
					"njsNBoP3zNxm5vbz5DzPJyKiG0T0Q0QetNZFVVVRq9V6eTweKwDodDoAAK11M",
					"R6PH4jo5AEAEZ2cc7da60IpBWMMjDFQSkFrXTjnbono9E9hIjIRkagsyyxN00",
					"2appuyLDMRiURkctE99UBEH/6AH5MkMUEQfB+NRu/+d/4NNv2kYBUI6xQAAAA",
					"ASUVORK5CYII=",
				"');",
				"width:12px;height:12px;",
				"position:relative;",
				"top:1px;",
				"cursor:pointer",
			"}",
			".hnh-dialog {",
				"position:absolute;",
				"z-index:10;",
				"top:30px;",
				"right:30px;",
				"width:200px;",
				"background:lightgrey;",
				"border:1px solid grey;",
				"border-radius:4px;",
				"padding:4px;",
				"color:black",
			"}",
			".hnh-dialog em {",
				"font-size:8px;",
				"margin-bottom:4px;",
			"}",
			".hnh-dialog label { font-size: 11px; }",
			".hnh-dialog input { position:relative;top:2px; }",
			".hnh-parentlink {",
				"text-decoration:underline",
			"}",
			".hnh-hover {",
				"position:fixed;",
				"background:white;",
				"border:1px solid darkgrey;",
				"padding:3px;",
			"}"
		].join(""));
		
		var selm = document.createElement("a");
		selm.className="hnh-settings";
		selm.onclick = function() {
			document.getElementById("hnh-settings-dialog").style.display = (document.getElementById("hnh-settings-dialog").style.display == "block") ? "none" : "block";
		};
		
		var t = document.getElementsByClassName("pagetop")[1];
		t.insertAdjacentHTML("afterbegin", " | ");
		t.insertAdjacentElement("afterbegin", selm);
		
		var d = document.createElement("div");
		d.id="hnh-settings-dialog";
		d.className="hnh-dialog";
		d.style.display = "none";
		d.innerHTML = '<strong>HNHance options</strong><br><em><a href="javascript:;" onclick="window.location.reload();">Reload</a> to see changes.</em><br>';
				
		for (var i in settings_enum) {
			var lb = document.createElement("label");
			var cb = document.createElement("input");
			cb.type = "checkbox";
			cb.onclick = (function(i) {
				return function() {
					var s = settings_get();
					s.features[i] = this.checked;
					settings_save(s);
				}
			}(i));
			
			if (settings_get().features[i]) cb.checked = 'checked';
			lb.appendChild(cb);
			lb.appendChild(document.createTextNode(settings_enum[i]));
			d.appendChild(lb);
			d.appendChild(document.createElement('br'));
		}
		
		d.insertAdjacentHTML("beforeend", "<strong>Custom themes</strong><br>");
		
		for (var i in settings_themes_enum) {
			var lb = document.createElement("label");
			var rb = document.createElement("input");
			rb.type = "radio";
			rb.name = "hnhance_custom_theme";
			rb.onclick = (function(i) {
				return function() {
					var s = settings_get();
					s.theme = i;
					settings_save(s);
				}
			}(i));
			
			if (settings_get().theme == i) rb.checked = 'checked';
			lb.appendChild(rb);
			lb.appendChild(document.createTextNode(settings_themes_enum[i]));
			d.appendChild(lb);
			d.appendChild(document.createElement('br'));
		}
		
		document.body.appendChild(d);
		
	}, false);
	
	/*!
	 * Feature: styleCommentLink
	 * Adds a visible a:visited color for comment links.
	 *
	 * @author mappy
	 * @version 0.1~hnhance1
	 */
	if (settings_get().features.styleCommentLink) window.addEventListener("load", function() {
	
		var css = [
			".subtext a:visited { ",
			"	color: #BCBCB8",
			"}"
		].join("\n");
		var dom_style = document.createElement("style");
		dom_style.appendChild(document.createTextNode(css));
		document.body.appendChild(dom_style);
		
	}, false);
	
	/*!
	 * Feature: showSubmissionsLink
	 * Adds an extra item to the top bar
	 *
	 * @author mappy
	 * @version 0.1~hnhance1
	 */
	if (settings_get().features.showSubmissionsLink) window.addEventListener("load", function() {
		var threadsLink = document.querySelectorAll(".pageTop a")[2];
		var unc = threadsLink.href.indexOf("threads");
		if (unc == -1) {
			// not logged in
		} else {
			var username = threadsLink.href.substr(unc + 11);
			threadsLink.insertAdjacentHTML(
				"afterend",
				" | <a href='"+document.location.protocol+"//news.ycombinator.com/submitted?id="+username+"'>submissions</a>"
			);
		}
	}, false);
	
	/*!
	 * Feature: confirmDownvotes
	 * Displays a confirmation dialog in case you accidentally fat-fingered the
	 *  downvote button.
	 *
	 * @author mappy
	 * @version 0.0~hnhance1
	 */
	if (settings_get().features.confirmDownvotes) window.addEventListener("load", function() {
		var as = document.querySelectorAll("a");
		for (var i in as) {
			if (! as[i].addEventListener) continue;
			if (as[i].id.indexOf("down_") !== 0) continue;
			
			as[i].className += "hnh-confirmation_required";
			as[i].setAttribute('onclick', 'return (confirm("Are you sure you want to downvote this post?") && vote(this));');
		}
	}, false);
	
	/*!
	 * Feature: hoverParents
	 * Displays a mouseover link to view the parent comment.
	 *
	 * @author mappy
	 * @version 1.0~hnhance1
	 */
	 
	if (settings_get().features.hoverParents) window.addEventListener("load", function() {
		
		// Determine chain of parents
		
		var rows = document.getElementsByTagName("table")[3].children[0].children;
		var last = {};
		for (var i in rows) {
			if (! rows[i].children) continue;
		
			try {
				rows[i].id = "hnpost_"+i;
				
				var width = rows[i].
					children[0].children[0].children[0].
					children[0].children[0].children[0].
					width;
				
				last[width] = i;
				
				if ((width-40) in last) {
					rows[i].setAttribute("data-hnparent", last[(width-40)]);
				}
			} catch (em_all) {}
		}
		last = {};
			
		// Create hover element
		
		var hover = document.createElement("div");
		//hover.id            = 'hnhance_tooltip';
		hover.style.display = 'none';
		hover.className     = "hnh-hover";
		document.body.appendChild(hover);
		
		hover.appendChild(document.createTextNode("loading..."));
		
		// Create parent links
		
		var u_tags = document.getElementsByTagName("u");
		for (var i = 0; i < u_tags.length; i++) {
			// Walk backward to <tr>. Root posts are one level higher
			var tr = u_tags[i].
				parentNode.parentNode.parentNode.parentNode.
				parentNode.parentNode.parentNode.parentNode;
			var post_id = (tr.id) ? tr.id : tr.parentNode.id;
			
			// Find parent post id
			var parent_id = document.getElementById(post_id).
				getAttribute("data-hnparent");
			
			if (! parent_id) continue;
			
			// Create parent link:
			var parent_link = document.createElement("a");
			parent_link.id = "hnparentlink_"+i;
			parent_link.className="hnh-parentlink";
			parent_link.setAttribute("href", "javascript:;");
			parent_link.appendChild(document.createTextNode("parent"));
			
			// Mouseover
			var mouseover = function(post_id, e) {
				while (hover.hasChildNodes()) {
					hover.removeChild(hover.lastChild);
				}
				// Retrieve
				var src = document.getElementById(post_id).
					children[0].children[0].children[0].
					children[0].children[2].children;
				for (var i = 0; i < src.length; i++) {
					hover.appendChild(src[i].cloneNode(true));
				}
				hover.style.display = 'block';
			};
			parent_link.onmouseover = mouseover.bind(
				mouseover, "hnpost_"+parent_id
			);
			
			// Mousemove
			parent_link.onmousemove = function(e) {
				hover.style.top = e.clientY + 10;
				hover.style.left = e.clientX + 10;
			}
			
			// Mouseout
			parent_link.onmouseout = function() {
				hover.style.display = 'none';
			};
			
			// Add to document
			u_tags[i].parentNode.appendChild(document.createTextNode(" | "));
			var u_el = document.createElement("u");
			//u_el.appendChild(parent_link);
			//u_tags[i].parentNode.appendChild(u_el);
			u_tags[i].parentNode.appendChild(parent_link);
		}
		
	}, false);
	
	/*!
	 * Feature: hoverProfiles
	 * Mouse over a user's name to view their profile stats.
	 *
	 * @author mappu
	 * @version 1.0~hnhance1
	 */
	if (settings_get().features.hoverProfiles) window.addEventListener("load", function() {
	
		var h_el = document.createElement("div");
		h_el.style.display = 'none';
		h_el.className     = "hnh-hover";
		document.body.appendChild(h_el);
		
		var a_els = document.getElementsByTagName("a");
		for (var i in a_els) {
			if (! a_els[i].href) continue;
			var upos = a_els[i].href.indexOf("user?");
			if (upos === -1) continue;
			
			a_els[i].addEventListener("mouseover", (function(url) {
				return function(e) {
					console.log("Getting '"+url+"'...");
					var xhr = new XMLHttpRequest();
					xhr.open("GET", url);
					xhr.onload = function() {
						var global = xhr.responseText;
						h_el.innerHTML = global.replace(/[\r\n]/g, "").match(/<table border=0>(.*?)\/table>/g, "$1");
					}
					xhr.send();
					
					h_el.style.top = e.clientY + 10;
					h_el.style.left = e.clientX + 10;
					h_el.style.display = "block";
					h_el.innerHTML = "Loading...";
				};
			}(a_els[i].href)));
			
			a_els[i].addEventListener("mousemove", function(e) {
				h_el.style.top = e.clientY + 10;
				h_el.style.left = e.clientX + 10;
			});
			
			a_els[i].addEventListener("mouseout", function() {
				h_el.style.display = "none";
			});
			
			//a_els[i].setAttribute("data-username", a_els[i].href.substr(upos + 8));
		}
	});
	
	/*!
	 * Feature: hn4chan
	 * Display 4chan/g/ related threads on the homepage
	 *
	 * @author  anon
	 * @grant   none
	 * @version 1.0.0~hnhance2
	 */

	if (settings_get().features.show4chan) window.addEventListener("load", function() {
		
		// No point loading this mod on comment pages
		if (document.location.href.indexOf("/item?") !== -1) return;
		
		var xhr = new XMLHttpRequest();
		xhr.open("GET", document.location.protocol + "//api.4chan.org/g/catalog.json");

		xhr.onload = function() {
			var catalog = JSON.parse(xhr.responseText);

			[].slice.call(document.querySelectorAll("td.subtext > a[href^=item]"))
				.map(function(commentLink) {
					var itemLink = commentLink
						.parentNode.parentNode.previousSibling.querySelector("td.title > a");

					return {
						link:
							itemLink
								.getAttribute("href")
								.replace(/^http:\/\/(?:www.)?|^https:\/\/(?:www.)?|\/$/g, ""),
						comment:
							"news.ycombinator.com/" +
							commentLink.getAttribute("href"),
						elem: commentLink.parentNode,
					};
				})
				.forEach(function(item) {
					catalog.forEach(function(page) {
						page.threads
							.filter(function(thread) { return thread.com; })
							.forEach(function(thread) {
								var comment = thread.com
									.replace(/<wbr>|<br>/g, "")
									.replace(/&lt;/g, "<")
									.replace(/&gt;/g, ">")
									.replace(/&#039;/g, "'")
									.replace(/&quot;/g, "\"")
									.replace(/&amp;/g, "&");

								if (
									comment.indexOf(item.link) != -1 ||
									comment.indexOf(item.comment) != -1)
								{
									var a = document.createElement("a");
									a.setAttribute(
										"href",
										document.location.protocol + "//boards.4chan.org/g/res/" + thread.no
									);
									a.appendChild(document.createTextNode("4chan"));

									item.elem.appendChild(document.createTextNode(" | "));
									item.elem.appendChild(a);
								}
							});
					});
				});
		};

		xhr.send();

	}, false);
	
	/*!
	 * Feature: collapseComments
	 * Provides a little squared + or squared - icon which can be clicked to
	 *   hide a tree of comments in a main Hacker News thread.
	 *
	 * @author "@drostie <http://drostie.org>"
	 * @version 0.0~hnhance1
	 * @license Creative Commons Zero waiver/license <http://creativecommons.org/publicdomain/zero/1.0/>
	 */
	if (settings_get().features.collapseComments) window.addEventListener("load", function() {
		/* This user script was authored by Chris Drost of drostie.org. To the extent 
		 * possible by all laws in all countries, I hereby waive all copyright and any 
		 * related rights under the Creative Commons Zero waiver/license, which you 
		 * may read online at:
		 * 
		 *     http://creativecommons.org/publicdomain/zero/1.0/ 
		 * 
		 * This means that you may copy, distribute, modify, and use my script without 
		 * any fear of lawsuits from me. It also means that my script is provided with 
		 * NO WARRANTIES of any kind, so that I may have no fear of lawsuits from you.
		 *
		 * I always enjoy emails or messages about strange new places where my code 
		 * and writings are being used, but I place no obligation on anyone to notify 
		 * me of such things.
		 */
		
		var array = function(ls) {
			return Array.prototype.slice.call(ls, 0);
		}
		var repeat = function(element, selector, n) {
			return n <= 0 ? element : repeat(element[selector], selector, n - 1);
		}
		var ratings_from_comment = function(c) {
			return c.parentNode.previousSibling.firstChild;
		}
		var indent = function(b) {
			return repeat(b, 'firstChild', 6).width;
		}
		var children_of_comment = function(c) {
			var row = repeat(c, 'parentNode', 6),
				out = [], 
				indent_level = indent(row);
			var next = function() {
				row = row.nextSibling;
				return row !== null && row.getElementsByClassName("comment").length > 0 && 
						indent(row) > indent_level;
			}
			while (next()) {
				out.push(row);
			}
			return out;
		}
		var icon_hidden = "\u229E", // unicode: "squared plus"
			icon_shown = "\u229F", // unicode: "squared minus"
			comments = array(document.getElementsByClassName("comment"));

		if (document.getElementsByClassName("title").length === 0) {
			// comment thread page, first comment replaces the root article.
			comments_list = comments_list.slice(1);
		}
		comments.forEach(function (c) {
			var child_rows = children_of_comment(c),
				shown = true,
				link = document.createElement("div"),
				target = ratings_from_comment(c);
			if (child_rows.length === 0) {
				return; // no sense in letting you hide nonexistent comments.
			}
			link.style.cursor = "pointer";
			link.style.fontSize = "12pt";
			link.innerHTML = icon_shown;
			link.onclick = function () {
				if (shown) {
					child_rows.forEach(function (r) {
						if (!r.hidden_by) {
							r.hidden_by = c;
							r.style.display = "none";
						}
					});
					link.innerHTML = icon_hidden;
					shown = false;
				} else {
					child_rows.forEach(function (r) {
						if (r.hidden_by === c) {
							r.hidden_by = undefined;
							r.style.display = "block";
						}
					});
					link.innerHTML = icon_shown;
					shown = true;
				}
				return false;
			};
			target.appendChild(link);
		});
	});
	
	/*!
	 * Theme: default
	 * Make no changes to the page style.
	 */
	if (settings_get().theme == "default") (function() {
		// Fix inconsistencies only.
		/*
		injectStyles([
			".hnh-parentlink { text-decoration: underline !important; }"
		].join(""));
		*/
	});	
	
	/*!
	 * Theme: customStyle
	 * Adds custom CSS to lay out HN in a more modern way.
	 *
	 * Original author unknown (adapted from https://gist.github.com/kballenegger/5022493)
	 * Modifications:
	 * - Restyle <a> elements
	 * - Remove hotlink from subtlepatterns.com, bundle if chrome extension
	 * - Fix layout of nested comments on threads page
	 *
	 * @version 1.0~hnhance1
	 */
	 
	if (settings_get().theme == "modern") (function() {
		injectStyles([
			'.default { width:100%; }',
			'.comment p, .comment + p {',
				'line-height: 16px;',
			'}',
			'.comment u, .comment u a, .comment u a:visited, .comment + p u, .comment + p u a, .comment + p u a:visited {',
				'color: #777;',
				'text-decoration: none ;',
			'}',
			'.pagetop {',
				'font-family: "Lucida Grande", "Segoe UI", Arial, Helvetica, sans-serif;',
				'font-size: 14px;',
			'}',
			'.subtext {',
				'font-family: "Lucida Grande", "Segoe UI", Arial, Helvetica, sans-serif;',
				'height: 15px;',
				'vertical-align: top;',
			'}',
			'.title {',
				'color: #ccc;',
				'font-family: "Lucida Grande", "Segoe UI", Arial, Helvetica, sans-serif;',
				'font-size: 12px;',
			'}',
			'a {',
				'text-decoration: none;',
			'}',
			'a:hover {',
				'text-decoration: underline;',
			'}',
			'body {',
				'background-image: url('+(
					// http://subtlepatterns.com/patterns/groovepaper.png 
					(window.chrome && window.chrome.extension) ?
					window.chrome.extension.getURL('groovepaper.png') :
					'http://i.imgur.com/5lcKlW9.png'					
				)+');', 
				'margin: 0;',
			'}',
			'body > center > table {',
				'background-color: transparent;',
				'width: 100%;',
			'}',
			'body > center > table > tbody > tr > td > table {',
				'margin: 0 auto;',
				'width: 960px;',
			'}',
			'body > center > table > tbody > tr:first-child > td {',
				'background-image: -webkit-linear-gradient(top, #f60, #f70);',
				'border-bottom: solid 1px #f60;',
				'box-shadow: 0 2px rgba(0,0,0,.1);',
			'}',
			'body > center > table > tbody > tr:first-child > td > table {',
				'margin: 0 auto;',
				'padding: 10px;',
				'width: 960px;',
			'}',
			'body > center > table > tbody > tr:first-child > td > table > tbody > tr > td:first-child {',
				'padding-right: 10px;',
			'}',
			'body > center > table > tbody > tr:nth-child(3) > td {',
				'padding: 20px 0;',
			'}',
			'body > center > table > tbody > tr:nth-child(3) > td > form {',
				'margin: 0 auto;',
				'width: 960px;',
			'}',
			'body > center > table > tbody > tr:nth-child(3) > td > table {',
				'margin: 0 auto;',
				'width: 960px;',
			'}',
			'body > center > table > tbody > tr:nth-child(3) > td > table > tbody > tr:first-child > td:nth-child(2).title > a {',
				'font-size: 18px;',
			'}',
			'td > center > table {',
				'background-color: white;',
				'border: solid 1px #000;',
				'width: 940px;',
			'}',
			'td > center > table td {',
				'padding: 10px;',
			'}',
			'td, .comment, .default, .comhead, .yclinks, .dead {',
				'font-family: "Lucida Grande", "Segoe UI", Arial, Helvetica, sans-serif;',
			'}',
		].join(""));
	})();
	
	/*!
	 * Theme: GeorgifyMod
	 * Alternate CSS for Hacker News with a focus on typography and readability.
	 * Uses Georgia as the font and some typographic sugar to improve legibility. 
	 * 
	 * Original by @tuhin <http://userstyles.org/styles/46180>
	 * 
	 * HNHance1 modifications:
	 * - Increased width from 700px to accomodate extra header elements
	 * - Modified style to accomodate image link in header
	 * - Modified selectors to support parent hover links
	 * - Fixed bug causing layout issues with titled pages
	 * - Fixed layout flaw causing incorrect post nesting
	 *
	 * @version v1~hnhance1
	 * @author  "@tuhin"
	 * @url     http://userstyles.org/styles/46180
	 */
	if (settings_get().theme == 'georgify') injectStyles([
		'.pageTop font { display:none; }',
		'table {background-color: #ffffff!important; width:800px!important;}',
		'tbody tr td {background-color: #ffffff!important;}',
		'table tbody {background-color: #ffffff!important;}',
		'span.pagetop { color: #bbbbbb!important; padding-bottom:40px!important; font-family:Georgia, Times, serif!important; font-size:10pt!important; line-height: 10px!important; }',
		'span.pagetop a:link, span.pagetop a:visited  {font-family:Georgia, Times, serif!important; font-size:10pt!important; color:#bbbbbb!important; padding-left: 5px!important;padding-right: 5px!important; margin-bottom: 10px!important; padding-bottom: 4px!important; font-weight: normal!important;border-bottom: 0px dotted #fff!important; }',
		'span.pagetop a:hover, span.pagetop a:active {font-family:Georgia, Times, serif!important; font-size:10pt!important; color:#f60!important; '+/*padding-left: 5px!important;padding-right: 5px!important; margin-bottom: 10px!important; padding-bottom: 4px!important;'*/' font-weight: normal!important; border-bottom: 0px dotted #ffa86e!important; }',
		'body { font-family:Georgia, Times, serif!important; font-size:12pt!important; color:#222222!important; }',
		'td { font-family:Georgia, Times, serif!important; font-size:12pt!important; color:#999999!important; padding: 3px!important; }',
		'.admin td { font-family:Georgia, Times, serif!important; font-size:8.5pt!important; color:#222222!important; }',
		'td.subtext { font-family:Georgia, Times, serif!important; font-size:  12px!important; color:#999999!important; border-bottom: 1px dotted #ddd!important ; }',
		'form { margin-bottom: 0px!important; }',
		'input { font-family:Georgia, Times, serif!important; font-size:15px!important; color:#222222!important; padding: 8px 8px 8px 8px!important;',
		'background-color: #fff!important;',
		'color: #333!important;',
		'width: auto!important;',
		'font-family: Georgia, Times, serif!important;',
		'font-size: 15px!important;',
		'border: 1px solid #ccc!important;',
		'-moz-border-radius: 4px!important;',
		'-webkit-border-radius: 4px!important;',
		'border-radius: 4px!important;}',
		'input[type="text"] { font-family:Georgia, Times, serif!important; font-size:15px!important; color:#222222!important; padding: 8px 8px 8px 8px!important;',
		'background-color: #fff!important;',
		'color: #333!important;',
		'width: auto!important;',
		'font-family: Georgia, Times, serif!important;',
		'font-size: 15px!important;',
		'border: 1px solid #ccc!important;',
		'-moz-border-radius: 4px!important;',
		'-webkit-border-radius: 4px!important;',
		'border-radius: 4px!important;}',
		'input[type="submit"] { font-family:Georgia, Times, serif!important; color:#222222!important; padding: 8px 8px 8px 8px!important;',
		'background-color: #f60!important;',
		'color: #000!important;',
		'width: auto!important;',
		'font-family: Georgia, Times, serif!important;',
		'font-size: 13px!important;',
		'border: 1px solid #f60!important;',
		'-moz-border-radius: 4px!important;',
		'-webkit-border-radius: 4px!important;',
		'border-radius: 4px!important;',
		'cursor: pointer!important;}',
		'textarea { font-family:Georgia, Times, serif!important; font-size:15px!important; color:#222222!important; column-span: 200px!important; border: 1px solid #ccc!important;',
		'-moz-border-radius: 4px!important;',
		'-webkit-border-radius: 4px!important;',
		'border-radius: 4px!important; width: 660px!important; height: 100px!important;}',
		'a:link { color:#ffa86e!important; text-decoration:none!important; border-bottom: 1px dotted #fff!important; }',
		'a:hover, a:active { color:#ffa86e!important; text-decoration:none!important; border-bottom: 1px dotted #ffa86e!important; }',
		'a:visited { color:#aaaaaa!important; text-decoration:none!important; border-bottom: 1px dotted #fff}',
		'.default { font-family:Georgia, Times, serif!important; font-size: 12pt!important; color:#999999!important; border-bottom: 1px dotted #ccc!important; padding: 0px!important; width: 100%;}',
		'.admin { font-family:Georgia, Times, serif!important; font-size:8.5pt!important; color:#222222!important; }',
		'.title { font-family:Georgia, Times, serif!important; font-size: 12pt!important; color:#999999!important; padding-top: 12px!important; font-weight: normal!important;}',
		'.title a:link { font-weight: normal!important; font-family:Georgia, Times, serif!important; font-size: 14pt!important; color:#222222!important; margin-right: 6px; }',
		'.title a:hover, .title a:active { border-bottom: 1px dotted #ffa86e!important; }',
		'.title a:visited { color:#aaaaaa; border-bottom:1px dotted #fff;}',
		'.adtitle { font-family:Georgia, Times, serif!important; font-size:  9pt!important; color:#999999!important; }',
		'.yclinks { font-family:Georgia, Times, serif!important; font-size:  8pt!important; color:#999999!important;  padding-top: 15px!important; }',
		'.yclinks a:link, .yclinks a:visited { color:  #bbb!important; padding-left: 5px!important; padding-right: 5px!important; font-weight: normal!important; border-bottom: none!important; }',
		'.yclinks a:hover, .yclinks a:active {color:  #f60!important; padding-left: 5px!important; padding-right: 5px!important; font-weight: normal!important; border-bottom: none!important;}',
		'.pagetop { font-family:Georgia, Times, serif!important; font-size: 12pt!important; color:#222222!important; }',
		'.comhead { font-family:Georgia, Times, serif!important; font-size:  9pt!important; color:#888888!important; }',
		'.comment { color:  #222222!important;font-family:Georgia, Times, serif!important; font-size:  15px!important; line-height: 22px!important; padding-top: 10px!important; }',
		'.dead { font-family:Georgia, Times, serif!important; font-size:  9pt!important; color:#dddddd!important; }',
		'i { font-style: italic!important; }',
		'u, .hnh-parentlink { text-decoration: none!important; border-bottom: 1px dotted #ffa86e!important; font-size: 12px!important; margin-top: -10px!important; }',
		'.comment a:link, .comment a:visited  { color: #ffa86e!important; border-bottom: 1px dotted #fff!important; font-weight:normal!important;}',
		'.comment a:hover, .comment a:active {color: #ffa86e!important; border-bottom: 1px dotted #ffa86e!important; font-weight:normal!important;}',
		'.dead a:link, .dead a:visited { color:#bbbbbb!important; }',
		'.pagetop a:visited { color:#222222!important;}',
		'.topsel a:link, .topsel a:visited { color:#ffffff!important; }',
		'.subtext { font-family:Georgia, Times, serif!important; font-size:  8pt!important; color:#999999!important; padding-bottom:20px!important; font-style: italic!important; }',
		'.subtext a:link { padding-left:2px!important ; padding-right:4px!important; color: #ffa86e!important ; font-weight: normal!important; border-bottom: 1px dotted #fff!important;}',
		'.subtext a:hover, .subtext a:active { text-decoration:none!important; border-bottom: 1px dotted #ffa86e!important;}',
		'.subtext a:visited { padding-left:2px!important ; padding-right:4px!important; color: #ffa86e!important ; font-weight: normal!important; border-bottom: 1px dotted #fff; }',
		'.comhead a:link  { padding-left:2px!important ; padding-right:4px!important; color: #ffa86e!important ; font-weight: normal!important; border-bottom: 1px dotted #fff!important; }',
		'.comhead a:hover, .comhead a:active  { padding-left:2px!important; padding-right:4px!important; color: #ffa86e!important ; font-weight: normal!important; text-decoration:none!important; border-bottom: 1px dotted #ffa86e!important; }',
		'.default p { margin-top: 2px!important; margin-bottom: 0px!important; padding-bottom: 5px;padding-top: 15px!important; text-rendering:optimizeLegibility!important; }',
		'.pagebreak {page-break-before:always!important;}',
		'pre { overflow: auto!important; padding: 2px!important; max-width:600px!important; }',
		'pre:hover {overflow:auto!important;}',
	].join(""));
	
})();